package com.company;

public class Player {
    // a two dimensional array that saves the player's coordinate
    private byte[][] playerCoordinate;

    /**
     * Creates a player with it's "playerCoordinate"
     */
    public Player(){
        playerCoordinate = new byte[6][6];
    }

    /**
     * returns "playerCoordinate"
     * @return playerCoordinate
     */
    public byte[][] getarr() {
        return playerCoordinate;
    }

    /**
     * gets a player's array, a x and a y of a coordinate to check if this coordinate is a blank or not
     * @param arr the player's array
     * @param x the x of coordinate to check
     * @param y the y of coordinate to check
     * @return true if it's not blank and false if it is almost a blank
     */
    public boolean containsInArray(byte[][] arr, int x, int y){
        if (arr[x][y] != 0)
            return true;
        else
            return false;
    }

    /**
     * gets the opponent array of coordinates and a x and y coordinate to check
     * if it's blank, add a disc to the list of coordinates
     * @param arrOpponent array of opponent discs
     * @param x the x coordinate
     * @param y the y coordinate
     */
    public void addDisc(byte[][]arrOpponent, int x, int y){
        if ((!containsInArray(playerCoordinate, x, y)) && (!containsInArray(arrOpponent, x, y)))
            playerCoordinate[x][y]++;
    }

    /**
     * Checks if the match ends or not
     * @return true if the match already ends and false if the match should be continued
     */
    public boolean searchIfMatchEnds(){
        for (int i = 0; i < 6; i++){
            for (int j = 0; j < 6; j++){
                if (containsInArray(playerCoordinate, j, i)){
                    if (checkSides(j, i))
                        return true;
                }
            }
        }
        return false;
    }

    /**
     * gets x and y checks the side to count the number of discs in a line
     * @param x the x coordinate
     * @param y the y coordinate
     * @return if there is a possible line between a player's disc and there is 5 of them returns true
     */
    public boolean checkSides(int x, int y){
        if (x != 0){
            if (containsInArray(playerCoordinate, x - 1, y)){
                if (countTheMoves(x, y, -1, 0) == 5)
                    return true;
            }
            if (y != 0){
                if (containsInArray(playerCoordinate, x - 1, y - 1))
                    if (countTheMoves(x, y, -1, -1) == 5)
                        return true;
            }
            if (y != 5){
                if (containsInArray(playerCoordinate, x - 1, y + 1))
                    if (countTheMoves(x, y, -1, 1) == 5)
                        return true;
            }
        }
        if (x != 5){
            if (containsInArray(playerCoordinate, x + 1, y)){
                if (countTheMoves(x, y, 1, 0) == 5)
                    return true;
            }
            if (y != 0){
                if (containsInArray(playerCoordinate, x + 1, y - 1))
                    if (countTheMoves(x, y, 1, -1) == 5)
                        return true;
            }
            if (y != 5){
                if (containsInArray(playerCoordinate, x + 1, y + 1)) {
                    if (countTheMoves(x, y, 1, -1) == 5)
                        return true;
                }
            }
        }
        if (y != 0){
            if (containsInArray(playerCoordinate, x, y - 1))
                if (countTheMoves(x, y, 0, -1) == 5)
                    return true;
        }
        if (y != 5){
            if (containsInArray(playerCoordinate, x, y + 1)) {
                if (countTheMoves(x, y, 0, 1) == 5)
                    return true;
            }
        }
        return false;
    }

    /**
     * draws a line from x, y coordinate from a player's disc into
     * specified direction to check if there is 5 in a line returns 5
     * @param x the x of the coordinate
     * @param y the y of the coordinate
     * @param moveX the exact direct into x axis
     * @param moveY the exact direct into y axis
     * @return the number of discs of the player in a line
     */
    public int countTheMoves(int x, int y, int moveX, int moveY){
        int x1 = x + moveX;
        int y1 = y + moveY;
        int counter = 1;
        while ((counter < 5) && (x1 > -1) && (x1 < 6)
                && (y1 > -1) && (y1 < 6) && (containsInArray(playerCoordinate, x1, y1))){
            counter++;
            x1 += moveX;
            y1 += moveY;
        }
        return counter;
    }

    /**
     * displays the visualMap
     * @param arr the coordinates of player1's discs
     * @param arr2 the coordinates of player2's discs
     */
    public void display(byte[][] arr, byte[][]arr2){
        System.out.println("  0 1 2 3 4 5");
        for (int i = 0; i < 3; i++){
            System.out.print(i + "|");
            for (int j = 0; j < 6; j++){
                if (containsInArray(arr, j, i))
                    System.out.print("r");
                else {
                    if (containsInArray(arr2, j, i))
                        System.out.print("b");
                    else
                        //unicode of white circle
                        System.out.print('\u25CB');
                }
                if ((j == 2) || (j == 5))
                    System.out.print("|");
                else
                    System.out.print(" ");
            }
            System.out.println();
        }
        System.out.print("  ");
        for (int j = 1; j < 12; j++)
            System.out.print("_");
        System.out.println();
        for (int i = 3; i < 6; i++) {
            System.out.print(i + "|");
            for (int j1 = 0; j1 < 6; j1++) {
                if (containsInArray(arr, j1, i))
                    System.out.print("r");
                else {
                    if (containsInArray(arr2, j1, i))
                        System.out.print("b");
                    else
                        //unicode of white circle
                        System.out.print('\u25CB');
                }
                if ((j1 == 2) || (j1 == 5))
                    System.out.print("|");
                else
                    System.out.print(" ");
            }
            System.out.println();
        }
    }

    /**
     * gets the coordinate domain of the board from the "searchCoordinate" method and rotates 90 degrees clockwise
     * @param boardNumber the number of board to rotate
     */
    public void rotate(byte boardNumber){
        byte[] bytes = new byte[2];
        bytes = searchCoordinate(boardNumber);
        byte tmp = playerCoordinate[bytes[0] + 2][bytes[1]];
        playerCoordinate[bytes[0] + 2][bytes[1]] = playerCoordinate[bytes[0]][bytes[1]];
        byte tmp1 = playerCoordinate[bytes[0] + 2][bytes[1] + 2];
        playerCoordinate[bytes[0] + 2][bytes[1] + 2] = tmp;
        byte tmp2 = playerCoordinate[bytes[0]][bytes[1] + 2];
        playerCoordinate[bytes[0]][bytes[1] + 2] = tmp1;
        playerCoordinate[bytes[0]][bytes[1]] = tmp2;

        tmp = playerCoordinate[bytes[0] + 1][bytes[1]];
        playerCoordinate[bytes[0] + 1][bytes [1]] = playerCoordinate[bytes[0]][bytes[1] + 1];
        tmp1 = playerCoordinate[bytes[0] + 2][bytes[1] + 1];
        playerCoordinate[bytes[0] + 2][bytes[1] + 1] = tmp;
        tmp2 = playerCoordinate[bytes[0] + 1][bytes[1] + 2];
        playerCoordinate[bytes[0] + 1][bytes[1] + 2] = tmp1;
        playerCoordinate[bytes[0]][bytes[1] + 1] = tmp2;
    }

    /**
     * gets a board number and returns a byte array that the first
     * index is the x of board's coordinate and the last
     * index is the y of board's coordinate
     * @param boardNumber number of board to search
     * @return the byte array
     */
    public byte[] searchCoordinate(byte boardNumber){
        byte[] c = new byte[2];
        switch (boardNumber){
            case 1:
                c[0] = 0;
                c[1] = 0;
                break;
            case 2:
                c[0] = 3;
                c[1] = 0;
                break;
            case 3:
                c[0] = 0;
                c[1] = 3;
                break;
            case 4:
                c[0] = 3;
                c[1] = 3;
                break;
        }
        return c;
    }
}


